;##################################################################
;
;   Phoenix-Z80 (High scores)
;
;   Programmed by Patrick Davidson (pad@calc.org)
;        
;   This program is in the public domain.  There is no warranty.
;
;   This file was last updated October 1, 2000.
;
;##################################################################     

scoring_msg:
        .db     "Shields",0
        .db     "Speed",0
        .db     "Bonus",0
        .db     "Money",0
        .db     0
        .db     "Score",0
        .db     0
        .db     "  Press (ENTER)",0
        .db     -1

highscore_title:
        .db     "   High Scores",0

highscore_prompt:
        .db     " Enter your name",0

game_finished:
        ld      hl,completed
        ld      a,(hl)
        or      a
        jp      nz,no_high_score
        ld      (hl),1

        call    synchronize
        call    synchronize

        call    restore_memory

        bcall(_clrlcdf)
        ld      hl,scoring_msg
        call    display_hl_msgs

        ld      hl,$0B03
        ld      (currow),hl
        ld      hl,(player_cash)        ; max 15000
        bcall(_disphl)

        ld      hl,$0B02
        ld      (currow),hl
        ld      hl,(bonus_score)        ; max 15000
        bcall(_disphl)

        ld      hl,$0B01
        ld      (currow),hl
        ld      hl,(time_score)         ; max 19000
        bcall(_disphl)

        ld      a,(player_pwr)          ; max 16000
        ld      hl,0
        ld      de,1000
        or      a
        jr      z,shield_done
        ld      b,a
shield_addup:
        add     hl,de
        djnz    shield_addup
shield_done:
        push    hl
        ld      de,(time_score)
        add     hl,de
        ld      de,(bonus_score)
        add     hl,de
        ld      de,(player_cash)
        add     hl,de
        ld      (bonus_score),hl

        ld      de,$0B05
        ld      (currow),de
        bcall(_disphl)

        ld      hl,$0B00
        ld      (currow),hl
        pop     hl
        bcall(_disphl)

loop_show_score:
        bcall(_getcsc)
        cp      G_ENTER
        jr      nz,loop_show_score

;############## Check if you have high score

        ld      hl,(high_scores_end)            ; Check against lowest score
        ld      de,(bonus_score)
        bcall(_cphlde)
        jr      nc,no_high_score

        ld      (high_scores_end),de            ; Put your score in bottom
        ld      hl,high_scores+(13*6)
        ld      b,10
loop_space:
        ld      (hl),' '
        inc     hl
        djnz    loop_space
        
        ld      b,6                             ; Bubble it towards the top
        ld      de,high_scores_end              ; DE -> entry to move up
loop_bubble:                                    
        ld      hl,-13
        add     hl,de                           ; HL -> entry to compare with

        push    de

        bcall(_ldhlind)                       ; HL = score above this one
        push    hl
        ex      de,hl
        ld      e,(hl)
        inc     hl
        ld      d,(hl)                          ; DE = this score
        pop     hl
        bcall(_cphlde)
        pop     de                              ; DE -> this entry
        jr      nc,no_bubble_up

        inc     de                              ; DE -> very end of entry
        ld      hl,-13
        add     hl,de                           ; HL -> previous entry
        push    bc
        ld      b,13
loop_exchange:
        ld      a,(de)
        ld      c,a
        ld      a,(hl)
        ld      (de),a
        ld      (hl),c
        dec     hl
        dec     de
        djnz    loop_exchange
        pop     bc
        dec     de                              ; DE -> previous entry

        djnz    loop_bubble

no_bubble_up:
        ld      hl,-11
        add     hl,de

        push    hl
        push    bc
        call    display_high_scores
        ld      hl,0
        ld      (currow),hl
        ld      hl,highscore_prompt
        bcall(_puts)
        pop     bc
        pop     hl
        inc     b
        ld      c,b
        ld      b,0
        ld      (currow),bc
        call    input_name

;############## Show high scores

no_high_score:
        call    display_high_scores
        ld      hl,0
        ld      (currow),hl
        ld      hl,highscore_title
        bcall(_puts)
        call    loop_show_highs
        jp      restart

        bcall(_getcsc)

loop_show_highs:
        bcall(_getcsc)
        cp      6
        jr      c,loop_show_highs
        ret

;############## Prompt for name entry

input_name:
        push    hl
        pop     ix
        ld      b,0
enter_name_loop:
        bcall(_getcsc)
        or      a
        jr      z,enter_name_loop
        cp      G_DEL
        jr      z,backup
        cp      G_ENTER
        ret     z
        ld      c,a
        ld      a,10
        cp      b
        jr      z,enter_name_loop
        ld      hl,chartable-10
        ld      e,c
        ld      d,0
        add     hl,de
        ld      a,(hl)
        ld      (ix),a
        bcall(_putc)
        inc     b
        inc     ix
        jr      enter_name_loop
backup: dec     b
        jr      c,toofar
        dec     ix
        ld      (ix),32
        ld      hl,curcol
        dec     (hl)
        ld      a,32
        bcall(_putc)
        dec     (hl)
        jr      enter_name_loop
toofar: inc     b
        jr      enter_name_loop

chartable:
        .db     ":WRMH."
        .db     "..0VQLG!..ZUPKFC"
        .db     "..YTOJEBX.>SNIDA"
        .db     ".12345.."

;############## Display the high score table

display_high_scores:
        call    restore_memory
        bcall(_clrlcdf)
        bcall(_getcsc)

        ld      hl,high_scores
        ld      b,7
        ld      a,1
        ld      (currow),a
loop_display_hs:
        xor     a
        ld      (curcol),a
        push    hl
        bcall(_puts)
        pop     hl
        ld      de,11
        add     hl,de
        push    hl
        bcall(_ldhlind)
        ld      a,$b
        ld      (curcol),a
        bcall(_disphl)
        pop     hl
        inc     hl
        inc     hl
        djnz    loop_display_hs
        ret
